Imports System.IO
Imports System.Linq
Imports VideoRecorder.Shared
Imports Windows.Foundation
Imports Windows.Foundation.Collections
Imports Windows.Media.Capture
Imports Windows.Storage.Streams
Imports Windows.UI.Xaml
Imports Windows.UI.Xaml.Controls
Imports Windows.UI.Xaml.Controls.Primitives
Imports Windows.UI.Xaml.Data
Imports Windows.UI.Xaml.Input
Imports Windows.UI.Xaml.Media
Imports Windows.UI.Xaml.Navigation
Imports System.Collections.Generic
Imports System

' The Basic Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=234237

Namespace VideoRecorder.W8
    ''' <summary>
    ''' A basic page that provides characteristics common to most applications.
    ''' </summary>
    Partial Public NotInheritable Class MainPage
        Inherits VideoRecorder.W8.Common.LayoutAwarePage

        ' For managing button and application state.
        Private Enum ButtonState
            Initialized
            Ready
            Recording
            Playback
            Paused
            NoChange
            CameraNotSupported
        End Enum

        Private currentAppState As ButtonState

        Private _videoRecorder As VideoRecorderCore

        Public Sub New()
            Me.InitializeComponent()

            BottomAppBar.IsOpen = True
            VideoRecorderCore.Instance = New W8VideoRecorder
            _videoRecorder = VideoRecorderCore.Instance
        End Sub

        ''' <summary>
        ''' Invoked when this page is about to be displayed in a Frame.
        ''' </summary>
        ''' <param name="e">Event data that describes how this page was reached.  The Parameter
        ''' property is typically used to configure the page.</param>
        Protected Overrides Sub OnNavigatedTo(ByVal e As NavigationEventArgs)
            InitializeVideoRecorder()
        End Sub

        Protected Overrides Sub OnNavigatedFrom(ByVal e As NavigationEventArgs)
            ' Dispose of camera and media objects.
            DisposeVideoPlayer()
            DisposeVideoRecorder()

            MyBase.OnNavigatedFrom(e)
        End Sub

        ' Update the buttons and text on the UI thread based on app state.
        Private Async Sub UpdateUI(ByVal currentButtonState As ButtonState, ByVal statusMessage As String)
            ' Run code on the UI thread.
            ' When the camera is not supported by the device.
            ' First launch of the application, so no video is available.
            ' Ready to record, so video is available for viewing.
            ' Video recording is in progress.
            ' Video playback is in progress.
            ' Video playback has been paused.
            ' Display a message.
            ' Note the current application state.
            Await Dispatcher.RunAsync(Windows.UI.Core.CoreDispatcherPriority.Low, Sub()
                                                                                      Select Case currentButtonState
                                                                                          Case ButtonState.CameraNotSupported
                                                                                              StartRecording.IsEnabled = False
                                                                                              StopPlaybackRecording.IsEnabled = False
                                                                                              StartPlayback.IsEnabled = False
                                                                                              PausePlayback.IsEnabled = False
                                                                                          Case ButtonState.Initialized
                                                                                              StartRecording.IsEnabled = True
                                                                                              StopPlaybackRecording.IsEnabled = False
                                                                                              StartPlayback.IsEnabled = False
                                                                                              PausePlayback.IsEnabled = False
                                                                                          Case ButtonState.Ready
                                                                                              StartRecording.IsEnabled = True
                                                                                              StopPlaybackRecording.IsEnabled = False
                                                                                              StartPlayback.IsEnabled = True
                                                                                              PausePlayback.IsEnabled = False
                                                                                          Case ButtonState.Recording
                                                                                              StartRecording.IsEnabled = False
                                                                                              StopPlaybackRecording.IsEnabled = True
                                                                                              StartPlayback.IsEnabled = False
                                                                                              PausePlayback.IsEnabled = False
                                                                                          Case ButtonState.Playback
                                                                                              StartRecording.IsEnabled = False
                                                                                              StopPlaybackRecording.IsEnabled = True
                                                                                              StartPlayback.IsEnabled = False
                                                                                              PausePlayback.IsEnabled = True
                                                                                          Case ButtonState.Paused
                                                                                              StartRecording.IsEnabled = False
                                                                                              StopPlaybackRecording.IsEnabled = True
                                                                                              StartPlayback.IsEnabled = True
                                                                                              PausePlayback.IsEnabled = False
                                                                                          Case Else
                                                                                      End Select
                                                                                      txtDebug.Text = statusMessage
                                                                                      currentAppState = currentButtonState
                                                                                  End Sub)
        End Sub

        Public Async Sub InitializeVideoRecorder()
            Await _videoRecorder.InitializeVideoRecorderAsync
            AddHandler _videoRecorder.CaptureFailed, AddressOf OnCaptureFailed

            ' Initialize the camera if it exists on the device.
            If _videoRecorder.VideoCaptureDevice IsNot Nothing Then
                ViewFinder.Source = TryCast(_videoRecorder.VideoCaptureDevice, MediaCapture)

                ' Start video capture and display it on the viewfinder.
                Await _videoRecorder.StartVideoSourceAsync

                ' Set the button state and the message.
                UpdateUI(ButtonState.Initialized, "Tap record to start recording...")
            Else
                ' Disable buttons when the camera is not supported by the device.
                UpdateUI(ButtonState.CameraNotSupported, "A camera is not supported on this device.")
            End If
        End Sub

        ' Set recording state: start recording.
        Private Async Sub StartVideoRecording()
            Try
                Await _videoRecorder.StartVideoRecordingAsync

                ' Set the button states and the message.
                UpdateUI(ButtonState.Recording, "Recording...")

                ' If recording fails, display an error.
            Catch e As Exception
                Me.Dispatcher.RunAsync(Windows.UI.Core.CoreDispatcherPriority.Normal, Sub() txtDebug.Text = "ERROR: " & e.Message.ToString)
            End Try
        End Sub

        ' Set the recording state: stop recording.
        Private Async Sub StopVideoRecording()
            Try
                ' Stop recording.
                If _videoRecorder.VideoCaptureDevice IsNot Nothing AndAlso _videoRecorder.IsVideoSourceStarted Then
                    Await _videoRecorder.StopVideoRecordingAsync

                    ' Set the button states and the message.
                    UpdateUI(ButtonState.NoChange, "Preparing viewfinder...")

                    StartVideoPreview()
                End If
                ' If stop fails, display an error.
            Catch e As Exception
                Me.Dispatcher.RunAsync(Windows.UI.Core.CoreDispatcherPriority.Normal, Sub() txtDebug.Text = "ERROR: " & e.Message.ToString)
            End Try
        End Sub

        ' Set the recording state: display the video on the viewfinder.
        Private Async Sub StartVideoPreview()
            Try
                ' Display the video on the viewfinder.
                If _videoRecorder.VideoCaptureDevice IsNot Nothing AndAlso _videoRecorder.IsVideoSourceStopped Then
                    ViewFinder.Source = TryCast(_videoRecorder.VideoCaptureDevice, MediaCapture)

                    Await _videoRecorder.StartVideoSourceAsync

                    ' Set the button states and the message.
                    UpdateUI(ButtonState.Ready, "Ready to record.")
                End If
                ' If preview fails, display an error.
            Catch e As Exception
                Me.Dispatcher.RunAsync(Windows.UI.Core.CoreDispatcherPriority.Normal, Sub() txtDebug.Text = "ERROR: " & e.Message.ToString)
            End Try
        End Sub

        ' record
        Private Sub StartRecording_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
            ' Avoid duplicate taps.
            StartRecording.IsEnabled = False

            StartVideoRecording()
        End Sub

        ' stop recording
        Private Sub StopPlaybackRecording_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
            ' Avoid duplicate taps.
            StopPlaybackRecording.IsEnabled = False

            ' Stop during video recording.
            If currentAppState = ButtonState.Recording Then
                StopVideoRecording()

                ' Set the button state and the message.
                UpdateUI(ButtonState.NoChange, "Recording stopped.")

                ' Stop during video playback.
            Else
                ' Remove playback objects.
                DisposeVideoPlayer()

                StartVideoPreview()

                ' Set the button state and the message.
                UpdateUI(ButtonState.NoChange, "Playback stopped.")
            End If
        End Sub

        ' play
        Private Async Sub StartPlayback_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
            ' Avoid duplicate taps.
            StartPlayback.IsEnabled = False

            ' Start video playback when the file stream exists.
            If _videoRecorder.VideoFileStream IsNot Nothing Then
                VideoPlayer.Play()
                ' Start the video for the first time.
            Else
                Await _videoRecorder.StopVideoSourceAsync

                'TODO: Hide ViewFinder so we can go full screen on playback
                ViewFinder.Source = Nothing

                Await _videoRecorder.InitializePlayerFileStreamAsync

                ' if we were sharing a ViewModel with Windows Phone, we would move VideoFile into the abstract base class to prevent casting
                VideoPlayer.SetSource(TryCast(_videoRecorder.VideoFileStream, IRandomAccessStream), (TryCast(_videoRecorder, W8VideoRecorder)).VideoFile.ContentType)

                ' Add an event handler for the end of playback.
                AddHandler VideoPlayer.MediaEnded, AddressOf VideoPlayerMediaEnded

                ' Start video playback.
                VideoPlayer.Play()
            End If

            ' Set the button state and the message.
            UpdateUI(ButtonState.Playback, "Playback started.")
        End Sub

        ' Pause video playback.
        Private Sub PausePlayback_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
            ' Avoid duplicate taps.
            PausePlayback.IsEnabled = False

            ' If mediaElement exists, pause playback.
            If VideoPlayer IsNot Nothing Then
                VideoPlayer.Pause()
            End If

            ' Set the button state and the message.
            UpdateUI(ButtonState.Paused, "Playback paused.")
        End Sub

        Private Sub DisposeVideoPlayer()
            If VideoPlayer IsNot Nothing Then
                ' Stop the VideoPlayer MediaElement.
                VideoPlayer.Stop()

                ' Remove playback objects.
                VideoPlayer.Source = Nothing

                _videoRecorder.DisposeVideoPlayer()

                ' Remove the event handler.
                RemoveHandler VideoPlayer.MediaEnded, AddressOf VideoPlayerMediaEnded
            End If
        End Sub

        Private Async Sub DisposeVideoRecorder()
            If _videoRecorder.VideoSource IsNot Nothing Then
                ' Stop captureSource if it is running.
                If _videoRecorder.VideoCaptureDevice IsNot Nothing AndAlso _videoRecorder.IsVideoSourceStarted Then
                    Await _videoRecorder.StopVideoSourceAsync
                End If

                ' Remove the event handlers for capturesource and the shutter button.
                RemoveHandler _videoRecorder.CaptureFailed, AddressOf OnCaptureFailed

                ' Remove the video recording objects.

                _videoRecorder.DisposeVideoRecorder()
                ViewFinder.Source = Nothing
            End If
        End Sub

        Private Sub OnCaptureFailed(ByVal sender As Object, ByVal e As GeneralErrorEventArgs)
            Me.Dispatcher.RunAsync(Windows.UI.Core.CoreDispatcherPriority.Normal, Sub() txtDebug.Text = "ERROR: " & e.ErrorMessage.ToString)
        End Sub

        ' Display the viewfinder when playback ends.
        Public Sub VideoPlayerMediaEnded(ByVal sender As Object, ByVal e As RoutedEventArgs)
            ' Remove the playback objects.
            DisposeVideoPlayer()

            StartVideoPreview()
        End Sub

        ''' <summary>
        ''' Populates the page with content passed during navigation.  Any saved state is also
        ''' provided when recreating a page from a prior session.
        ''' </summary>
        ''' <param name="navigationParameter">The parameter value passed to
        ''' <see cref="Frame.Navigate(Type, Object)"/> when this page was initially requested.
        ''' </param>
        ''' <param name="pageState">A dictionary of state preserved by this page during an earlier
        ''' session.  This will be null the first time a page is visited.</param>
        Protected Overrides Sub LoadState(ByVal navigationParameter As Object, ByVal pageState As Dictionary(Of String, Object))
        End Sub

        ''' <summary>
        ''' Preserves state associated with this page in case the application is suspended or the
        ''' page is discarded from the navigation cache.  Values must conform to the serialization
        ''' requirements of <see cref="SuspensionManager.SessionState"/>.
        ''' </summary>
        ''' <param name="pageState">An empty dictionary to be populated with serializable state.</param>
        Protected Overrides Sub SaveState(ByVal pageState As Dictionary(Of String, Object))
        End Sub
    End Class
End Namespace
